==> core-1.1.50.tgz: package/ecmascript/transforms/src/compat/es3/prop_lits.rs <==
use crate::util::is_valid_ident;
use swc_common::{Fold, FoldWith};
use swc_ecma_ast::*;

/// babel: `transform-property-literals`
///
/// # Input
/// ```js
/// var foo = {
///   // changed
///   "bar": function () {},
///   "1": function () {},
///
///   // not changed
///   "default": 1,
///   [a]: 2,
///   foo: 1
/// };
/// ```
///
/// # Output
/// ```js
/// var foo = {
///   bar: function () {},
///   1: function () {},
///
///   "default": 1,
///   [a]: 2,
///   foo: 1
/// };
/// ```
#[derive(Default, Clone, Copy)]
pub struct PropertyLiteral;

noop_fold_type!(PropertyLiteral);

impl Fold<Module> for PropertyLiteral {
    fn fold(&mut self, node: Module) -> Module {
        validate!(node.fold_children(self))
    }
}

impl Fold<PropName> for PropertyLiteral {
    fn fold(&mut self, n: PropName) -> PropName {
        let n = validate!(n.fold_children(self));

        match n {
            PropName::Str(Str {
                value: sym, span, ..
            }) => {
                if sym.is_reserved_for_es3() || !is_valid_ident(&sym) {
                    PropName::Str(Str {
                        span,
                        value: sym,
                        has_escape: false,
                    })
                } else {
                    PropName::Ident(Ident::new(sym, span))
                }
            }
            PropName::Ident(i) => {
                let Ident { sym, span, .. } = i;
                if sym.is_reserved_for_es3() || sym.contains('-') || sym.contains('.') {
                    PropName::Str(Str {
                        span,
                        value: sym,
                        has_escape: false,
                    })
                } else {
                    PropName::Ident(Ident { span, sym, ..i })
                }
            }
            _ => n,
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    test!(
        ::swc_ecma_parser::Syntax::default(),
        |_| PropertyLiteral,
        babel_basic,
        r#"var foo = {
  // changed
  "bar": function () {},
  "1": function () {},

  // not changed
  "default": 1,
  [a]: 2,
  foo: 1
};"#,
        r#"var foo = {
  bar: function () {},
  '1': function () {},

  "default": 1,
  [a]: 2,
  foo: 1
};"#,
        ok_if_code_eq
    );

    test!(
        ::swc_ecma_parser::Syntax::default(),
        |_| PropertyLiteral,
        str_lit,
        r#"'use strict';
var x = {
    'foo.bar': true
};"#,
        r#"'use strict';
var x = {
    'foo.bar': true
};"#,
        ok_if_code_eq
    );
}

==> core-1.2.14.tgz: package/ecmascript/transforms/src/compat/es3/prop_lits.rs <==
use crate::util::is_valid_ident;
use swc_ecma_ast::*;
use swc_ecma_visit::{Fold, FoldWith};

/// babel: `transform-property-literals`
///
/// # Input
/// ```js
/// var foo = {
///   // changed
///   "bar": function () {},
///   "1": function () {},
///
///   // not changed
///   "default": 1,
///   [a]: 2,
///   foo: 1
/// };
/// ```
///
/// # Output
/// ```js
/// var foo = {
///   bar: function () {},
///   1: function () {},
///
///   "default": 1,
///   [a]: 2,
///   foo: 1
/// };
/// ```
#[derive(Default, Clone, Copy)]
pub struct PropertyLiteral;

noop_fold_type!(PropertyLiteral);

impl Fold for PropertyLiteral {
    fn fold_prop_name(&mut self, n: PropName) -> PropName {
        let n = validate!(n.fold_children_with(self));

        match n {
            PropName::Str(Str {
                value: sym, span, ..
            }) => {
                if sym.is_reserved_for_es3() || !is_valid_ident(&sym) {
                    PropName::Str(Str {
                        span,
                        value: sym,
                        has_escape: false,
                    })
                } else {
                    PropName::Ident(Ident::new(sym, span))
                }
            }
            PropName::Ident(i) => {
                let Ident { sym, span, .. } = i;
                if sym.is_reserved_for_es3() || sym.contains('-') || sym.contains('.') {
                    PropName::Str(Str {
                        span,
                        value: sym,
                        has_escape: false,
                    })
                } else {
                    PropName::Ident(Ident { span, sym, ..i })
                }
            }
            _ => n,
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    test!(
        ::swc_ecma_parser::Syntax::default(),
        |_| PropertyLiteral,
        babel_basic,
        r#"var foo = {
  // changed
  "bar": function () {},
  "1": function () {},

  // not changed
  "default": 1,
  [a]: 2,
  foo: 1
};"#,
        r#"var foo = {
  bar: function () {},
  '1': function () {},

  "default": 1,
  [a]: 2,
  foo: 1
};"#,
        ok_if_code_eq
    );

    test!(
        ::swc_ecma_parser::Syntax::default(),
        |_| PropertyLiteral,
        str_lit,
        r#"'use strict';
var x = {
    'foo.bar': true
};"#,
        r#"'use strict';
var x = {
    'foo.bar': true
};"#,
        ok_if_code_eq
    );
}
